Newsgroups: comp.databases.xbase.misc,alt.comp.databases.xbase.clipper
Subject: CA-Clipper Frequently Asked Questions 1.01
Followup-To: poster
Summary: Frequently Asked Questions and Answers about the CA-Clipper
+        compiler and programming language

Archive-name: ca-clipper-faq/part3
Last-modified: 1994/04/23
Version: 1.01


             FREQUENTLY-ASKED-QUESTIONS (FAQ) LIST FOR CA-CLIPPER
                                  PART 3 OF 3
                 Copyright (C) 1992, 1994 by Mark W. Schumann

======================================================================
USE OF C AND ASSEMBLY LANGUAGE WITH CA-CLIPPER.
======================================================================

Q. Can you combine Borland C/C++ or Turbo C/C++ code with your
   CA-Clipper applications?

A. Yes, if you follow these guidelines:

   1. Use large model only (the -ml command-line compiler switch).
   2. Use no floating point operations, and disable them with
       the -f- switch.
   3. Use no Borland library functions; you may use standard C
       functions if they are found in CLIPPER.LIB or LLIBCE.LIB.
   4. You may leave register variables on and/or optimize
       for registers/jumps/speed.
   5. You may use the -v switch to include debugging information
       in your object code for Turbo Debugger.
   6. Turn stack checking off.
   7. If you use inline assembly, push DS before doing anything
       and pop it when done, unless you are SURE your code doesn't
       do anything to affect DS.

   One workable set of switches seems to be:

                               -c -f- -ml -N-

   These switches work, delivering optimized C code:

                         -c -f- -ml -N- -Z -O -G -w

# Ed Boggan (72701,1720) Section 4 #158250 22-Aug-91
# Carl Nelson (70363,1027) Section 2 #172091 29-Oct-91
# Nick Pombra (71477,2077) Section 4 #156089 12-Aug-91
# Alex Matijaca (73657,3171) Section 2 #152148 24-Jul-91
# Mark Zurier (71240,1222) Section 4 #294108 2-Feb-93

----------------------------------------------------------------------

Q. Is there a way to recompile or relink the standard CA-Clipper
   libraries to make CA-Clipper fully compatible with the Borland
   compilers?

A. No.  Floating point function calls are too tightly coupled with
   the compiler's code generation to allow that to be done.

   It is possible to write BCD replacement functions for the very
   low-level functions that call the floating-point math
   libraries.  (They have names such as __DVADD() and can be found
   in the map file output from any decent linker.)  The problem
   with this approach is that any numeric _literals_ in the source
   code are already embedded into the object code as double-precision
   floats.  There does not appear to be a good way for a replacement
   math library to deal with the fact that sometimes it will be
   handed BCD numerics and float numerics at different times.

----------------------------------------------------------------------

Q. How about C++?

A. Any C++ function which interfaces to your CA-Clipper code must be
   declared consistently as

                       extern "C" CLIPPER func (void)
 
   This is an ANSI linkage specifier which turns off "name mangling."
   One disadvantage is that if a function name is overloaded of course
   you may only access one instance of that function name from your
   CA-Clipper code.  (Otherwise how could CA-Clipper tell them apart?)

   If your C++ code must call Extend API functions, then you should
   edit your copy of EXTEND.H or EXTEND.API to bracket all the
   function declarations with the following code:

#ifdef __cplusplus
extern "C" {
#endif

  /* Existing function declarations here. */

#ifdef __cplusplus
}
#endif


   C++ code which is not called directly from your CA-Clipper
   application can be safely left "mangled."  For Borland C++ users,
   the normal prohibitions on use of floating point math remain.

# Diane Lask (72357,3574) Section 4 #144114 17-Jun-91

----------------------------------------------------------------------

Q. How do I compile Microsoft C 5.1 code?

A. The correct compiler switches are /FPa /Gs /Oals /AL.  You may
   use floating point math without restriction.

----------------------------------------------------------------------

Q. What about Microsoft C 6.0?  And later?

A. Use the undocumented /Gh compiler switch and the LLIBCA.LIB library
   from Microsoft C 5.1.  The library is available on the CLIPGER
   forum on Compuserve.

   For those who do not read German fluently or who do not have
   regular access to Compuserve it might be easier to purchase
   the C 5.1 library from Microsoft as a "downgrade" from a
   licensed later version of their C/C++ compiler line.  The number
   to call, in the United States, is 800-426-9400.  At the recorded
   message press 2 to order the downgrade.  It costs $25.00, or
   $15.00 without printed documentation.  These prices only apply to
   registered users of Microsoft C or C++ compilers from release
   6.0 forward.

# Don Caton (70007,5454) Section 4 #233968 26-Jun-92

----------------------------------------------------------------------

Q. Can you use Turbo Debugger with the embedded C and assembler
   code in your CA-Clipper application?

A. Yes.  For best results, compile the C code with the -v switch,
   assemble with /zi, and Turbo-link with /v.  Then just fire up
   TD with the name of your executable and set a breakpoint at
   the beginning of your C or assembler function of interest.

   I have found that TD286, Borland's 80286 protected-mode debugger,
   works as well without crowding CA-Clipper code out of memory.
   Still you are probably better off testing your C/assembler code
   from a small "shell" CA-Clipper program to save run-time memory.

   It is difficult to use TD386 with a CA-Clipper program since the
   former cannot coexist with the EMM386 expanded memory driver,
   and the later runs better with expanded memory.

----------------------------------------------------------------------

Q. I want to write a C function that concatenates a list of
   strings whose lengths are unknown at compile-time.  How can I
   do this efficiently?

A. If you're concerned about performance, the best possible solution
   would be to define a maximum string length beforehand.  If you can
   do that, this piece of code, submitted by Brian Marasca
   (71230, 1650 on Compuserve), is pretty high-performance:
 
   #define MAXLEN 120   /* Set this to the maximum required length. */
 
   CLIPPER cStrCat()
   {
       char *p, *q, *buf ;
       int cx = 1, parms = _parinfo ( 0 ) ;

       q = buf = _xgrab (MAXLEN + 1) ;
 
       while (parms--)
       {
            p = _parc (cx++) ;
            while (*p) *q++ = *p++ ;
       }
       *q = '\0' ;       /* very important! */
 
       _retc (buf) ;
       _xfree (buf) ;
   }
 
# Brian Marasca (71230,1650) Section 2 #172316 30-Oct-91
 
----------------------------------------------------------------------

Q. I want to call my C or assembler function with a reference
   parameter in a string.  Why can't you just use the pointer returned
   by _parc() to modify a string parameter?

A. Ted Means (73067,3332) provided this explanation (somewhat edited)
   on Compuserve:

                          BEGIN Ted Means quotation

   Clipper 5.0 performed under-the-hood memory optimizations that
   sometimes resulted in two strings being assigned the same pointer.

   For example, if you did this:

       X := "Some string"
       Y := X

   In all likelihood Clipper would simply make a copy of the pointer
   rather than actually copying the string.  It sounds great in
   theory, because it saves memory.  But problems arose when a C or
   ASM routine attempted to modify X or Y by writing directly to the
   assigned memory.  Since the same pointer was assigned to two
   different memvars, they both got changed, even though that was
   not the intended effect.

   So with the release of Clipper 5.01, Nantucket graciously
   provided a method for accessing strings that is more convenient. 
   If you call __StorCLen with a null pointer *before* calling
   __ParC, it warns Clipper that you plan to directly modify a
   __ParC pointer and Clipper will take whatever steps are necessary
   to make it safe to do so.

   This gives the best of both worlds, in that Clipper can still
   perform its memory optimizations, but C and ASM programmers get
   the benefit of being able to directly modify strings.  This is
   all documented in the Release Notes, but briefly, if you need to
   write directly to a string, use the following sequence:

     1)  Call __ParCLen to get the string's length.

     2)  Call __StorCLen with the length and a null pointer.

     3)  Call __ParC to get a pointer to the string.  This
         pointer will be safe to write to.

   Writing directly to a __ParC pointer without using this method is
   not sanctioned by Nantucket and is likely to cause problems. 
   Note, though, that if you don't need to write to the string you
   can simply call __ParC by itself.

                           END Ted Means quotation

# Ted Means (73067,3332) Section 4 #151944 23-Jul-91

----------------------------------------------------------------------

Q. How do I call CA-Clipper code from C?  Can this even be done?

A. With versions prior to 5.2, you have to set up Clip's internal
   EVAL stack properly with _xpushf() and _xpopf().  Afterwards,
   call _xdo() to execute the CA-Clipper function.  The whole thing is
   undocumented, and it is discouraged by CA because it requires
   the use of internal functions which are not guaranteed to remain
   unchanged from one CA-Clipper version to the next.  Most people
   spend weeks poking around the symbol tables to get this to work.

   CA-Clipper 5.2c's well-documented Item API (Application Programming
   Interface) allows you to evaluate CA-Clipper expressions, and view
   or change CA-Clipper variables, directly from C code.  You can
   request this documentation free from CA if you have purchased
   CA-Clipper 5.2c.

   It appears to be very difficult if not impossible to call
   CA-Clipper code successfully from within a C "main" program.  The
   environment must be CA-Clipper calling C calling CA-Clipper for
   this stunt to work.

# Steve Larsen (76370,1532) Section 4 #176347 19-Nov-91
# Jobst Hensiek (76656,1606) Section 4 #177670 25-Nov-91
# Jobst Hensiek (76656,1606) Section 4 #177304 23-Nov-91


======================================================================
CA-CLIPPER AND OTHER ENVIRONMENTS.
======================================================================

Q. How do I port my existing CA-Clipper code to a Unix system?

A. Available products for converting CA-Clipper to Unix-compatible
   C code include:

   Firm        Product            Location
   ----        -------            --------

   COMCON      FlagShip           WorkGroup Solutions, Inc.
                                  3375 E. Montana Place
                                  Denver, CO 80210 USA
                                     800 234 7813 (USA only)
                                  +1 303 757 7813
                                  Internet:   jerry@gcs.com

   Desktop AI  dbXport            Fairfield, Connecticut USA
               Ecstasy (x2c)      Internet:   x2c%dtop@uunet.uu.net
                                  Compuserve: Rich Roth 76137,3727

   Recital     Recital            Massachussetts, USA
                                  +1 508 750 1066
                                  (in US: 800 USE-R4GE)


FlagShip claims to be "NOT a porting tool" but "in fact a full Unix
implemetation of CA-Clipper through version 5.2c."  A spokesman goes
on to say that "the fact that C intermediate code exists is useful for
some, but safe to ignore for most users."

Rectial "supports all major Xbase code including Summer '87," (which
seems to imply that it doesn't support CA-Clipper 5.0 code).

"X2c is a Clipper compiler that runs on UNIX and creates a complied C
program from your clipper program."

   C Libraries for using .DBF and index files include:

   Firm        Product            Location
   ----        -------            --------

   APIS
    Software   herCules           Frankfurt/Main, Germany

     Includes source code, understands NDX, MDX, NTX, and IDX
     files.  Supports DOS, Windows, OS/2, and UNIX/XENIX.

   Greenleaf   Greenleaf          16479 Dallas Parkway, Suite 570
    Software    Database          Dallas, Texas 75248 USA
                Library           +1 214 248 2561
                              FAX +1 214 248 7830

     Source code and Windows DLL included.
     Supports DOS, Windows, OS/2, and UNIX/XENIX.

   Sequiter    CodeBase and
    Software    CodeBase++


   Again, no endorsement is implied here.

# Mark Bolzern (71630,3505) Section 2 #426464 16-Mar-94
# Mark Bolzern (71630,3505) Section 2 #426907 17-Mar-94
# Barry Mavin (Recital) (70640,164) Section 2 #256229 17-Sep-92
# Nantucket GmbH (76670,2224) Section 1 #153090 29-Jul-91
# Guenter Schwaninger <schwan@rbg.informatik.th-darmstadt.de>
  1-Dec-92.
# Rich Roth (76137,3727) Section 1 #316454 8-Apr-93

----------------------------------------------------------------------


Q. How do I translate Lotus files into DBF format?

A. Check out Calc(DB) by Pinnacle and ClipWKS by Grumpfish.
   "The key difference between the two is that Calc(DB) is a
   spreadsheet engine that performs full calculations and macro
   execution on worksheets (or any other data set) and ClipWKS
   focuses primarilly on access-only of worksheet data without
   any data-integrity checking," says Jeff Jochum, whose company
   produces Calc(DB).

   You can also use the Lotus Translate utility, or even load
   the spreadsheet into Quattro Pro and then save it with a .DBF
   extension.

# Jeff Jochum [Successware] (76665,3367) Section 9 #177065 24-Nov-91


----------------------------------------------------------------------

Q. I want to read and write files on my IBM minicomputer(s): a
   System/38 or AS/400.  How can I accomplish this?

A. A product from Aladdin allows your CA-Clipper application to access
   native AS400 data files at the record level.  Call Aladdin at
   +27 11 642-4588 or Compuserve 73427,1172.  Technical support was
   promised to be available in Los Angeles, California USA beginning
   January 1992, but I have not verified this.

   As of July 1993 I have been informed that this can also be done
   with CAS400 by T.L. Creates at +1 209 946 0105, and with Clip2
   by Dabiwa, Ltd., at +1 215 692 8130.

# Brad H. Codd [Aladdin] (73427,1172) Section 2 #179605 02-Dec-91
# Brad H. Codd [Aladdin] (73427,1172) Section 2 #179909 04-Dec-91

======================================================================
THIRD-PARTY LIBRARIES AND UTILITIES.
======================================================================

Q. What's the big deal about this FoxPro Rushmore Technology?  I am
   thinking about switching from CA-Clipper to FoxPro!

A. Rushmore is not an index type, nor is it magic as the Fox people
   would have you believe.  It is simply a FoxPro feature that
   automatically figures out the most efficient way to use existing
   indexes in a query (LOCATE, LIST, anything with a FOR clause,
   etc.).

   Rushmore is handy for interactive queries, but most application
   developers can know ahead of time what lookups need to be done
   on their database files and can arrange for an optimized lookup
   the old-fashioned way with little effort.

   That's why the Rushmore test results look so good but are not very
   relevant to what many application developers really do.  If you
   are trying to write a very open-ended reporting program, on the
   other hand, it may be just the ticket.

----------------------------------------------------------------------

Q. Someone said I could access FoxPro-type indices.  What is the
   difference between FoxPro indices and CA-Clipper indices?  What
   gain can I get from doing that, and how?

A. First off, there are several advantages to using a FoxPro-like
   index scheme.

   The nice thing about .CDX files is that they can contain more
   than one "tag," each of which relates to a key value.  Therefore
   you can drop all your many CA-Clipper .NTX files by converting each
   to a "tag" within a single .CDX file.  This also lets you always
   open a file with USE <file> INDEX <file> with assurance that you
   got all the tags.

   Replaceable Database Drivers which I know to support FoxPro-like
   files are:

    * The SIx Driver by Successware
    * Comix
    * DBFCDX driver already included in CA-Clipper 5.2c

   SIx does not give you Rushmore lookups, but it is shipped with an
   add-in called MACH which does.

   For more information, send email to Loren Scott of Successware at
   70274,2677 on Compuserve.

   If want traditional CA-Clipper .NTX index files, but also want
   a "Rushmore" sort of feature, there is a library called ClipMore
   which claims to accomplish this.  ClipMore costs US $150 and is
   published by Loadstone, Inc./215 Barmount Drive/Columbia, South
   Carolina 29210 USA.  Loadstone's telephone number is +1 803 731
   9128.


======================================================================
MISCELLANEOUS.
======================================================================

Q. How do I print USA postal barcodes with my CA-Clipper application?

A. Try the C Gazette issue of Winter 1990-91 (Volume 5, Number 2) for
   details on the format.  If you know C the translation into
   CA-Clipper can't be very hard.  If you don't know C you will find
   the syntax embarrassingly similar to CA-Clipper anyway.


----------------------------------------------------------------------

Q. How do I dial a modem from within a CA-Clipper application?

A. The modem on a serial port can be looked at as just another file
   as far as output is concerned.  (This is a useful
   oversimplification.)  You just need to open the serial port as a
   file and write a string such as "ATDT123-4567" to send a Hayes-
   compatible dial command to the modem.  A function like this would
   do the job nicely:

FUNCTION dialphone (nPort, cNumber)
LOCAL nFile := FCREATE ("COM" + STR (nPort, 1))  // Open port
FWRITE (nFile, "ATDT"+cNumber+CHR(13)+CHR(10))   // Write ATDT command
FCLOSE (nFile)                                   // Close it
RETURN NIL

----------------------------------------------------------------------

Q. Is A.D. 2000 a leap year?

A. Yes.

======================================================================
GETTING HELP AND FINDING OUT MORE ABOUT CA-CLIPPER.
======================================================================

Q. What books can help me learn CA-Clipper?

A. Download the file BOOKS.LST from Library 0.

----------------------------------------------------------------------

Q. What bulletin boards are available for CA-Clipper developers?

A. A complete list may be found in CLPBBS.ZIP in Library 0.

   Here is the list of those I know about.  Please inform me of
   any others you may come across.

   Country       Telephone         bps
   -------       ---------         ---

   Australia
     Melbourne   +61 3 752 3935    2400 v32/42/42bis
                 +61 3 752 3931    9600 v42
                 Melbourne Clipper User Group
                 (Also 3:634/389 on Fidonet)

   Canada
     Ontario     +1 416 538 3107
                 Artful Applications, Inc.

   South Africa  +27 11 643-4201
                 Clipper Expert Group
                 (Also 5:7101/21 on Fidonet)

   United Kingdom
                 +44 065679477    14400 HST
                 Welsh Wizard
                 (Also 050/061 on GT Power)

                 +44 480 471465
                 The Dark Knight Returns

   United States
     California  +1 310 453 7705
                 "The AMULET" Dan Gutierrez
     New Jersey  +1 201 568 7293   9600
                 "The BOSS" Mark Seiden
     Maryland    +1 301 251 9206   9600
                 "Darwin" P.L. Olympia
     Texas       +1 817 695 0058   2400
                 Vernon Six
     California  +1 818 332 6580   2400
                 Sean Thakkar (76414,452 on Compuserve)


   Of course there is also Compuserve, truly the meeting place of
   the CA-Clipper developers' community.  2400-baud access costs at
   least $4.00 per hour in the United States and Canada, and much
   more elsewhere.  When you join Compuserve, type GO CLIPPER to
   join the Forum, where CA/Nantucket staff people and (I mean this
   very literally) hundreds of other CA-Clipper developers visit every
   day to check messages, ask questions, and offer help.  The CLIPPER
   forum also carries what must be the largest collection of public
   domain, shareware, and freeware of interest to CA-Clipper
   developers.

   For more information on joining Compuserve, please call (in the
   United States) 1-800-848-8980.  This is a voice-only number.
   I will be happy to provide phone numbers or other pointers for
   non-U.S. users.

   If you use the Usenet/Internet and need to write email to a
   Compuserve mailbox, simply replace the comma in the user's
   address with a period, and then append "@compuserve.com" to
   the result.  (Therefore you can reach my Compuserve mailbox
   73750,3527 as <73750.3527@compuserve.com> from Usenet.)

----------------------------------------------------------------------

Q. Is there a mailing list for CA-Clipper users on the
   Usenet/Internet/Bitnet?  Fidonet?  Others?

A. Yes.  You can subscribe by sending the message "SUBSCRIBE CLIPPER
   Your-Name-Here" to <LISTSERV@BRUFPB.BITNET>.  Send list
   contributions to <CLIPPER@BRUFPB.BITNET>; administrative requests
   go to <LISTSERV@BRUFPB.BITNET>. Do not ever send administrative
   requests to the list address.

   On Fidonet, check the CLIPPER echo.

   There is also an active CA-Clipper echo on the cozier but
   lesser-known GT Power network.

----------------------------------------------------------------------

Q. Is there a Usenet newsgroup on CA-Clipper?

A. Yes, if you consider the alt.* hierarchy to be part of Usenet,
   which purists do not.  It's alt.comp.databases.xbase.clipper.

   There is also quite a bit of Clipper discussion on
   comp.databases.xbase.misc since a vote to create a separate
   comp.databases.xbase.clipper failed.

----------------------------------------------------------------------
